##################################################
# Test::Suite
#
# 1998, schilli@perlmeister.com
##################################################

use File::Copy;
use File::Basename;
use Cwd;

package Test::Suite;

##################################################
sub new {                            # Constructor
##################################################
    my $class = shift;

    my $self  = {};
    bless($self, $class);
}

##################################################
sub register {                  # Register scripts
##################################################
    my ($self, $testname, @scripts) = @_;

    push(@{$self->{tests}}, [$testname, @scripts]);
}

##################################################
sub update {          # Update all reference files
##################################################
    my ($self) = @_;

    foreach $test (@{$self->{tests}}) {    
        my ($testname, @scripts) = @$test;

        foreach $script (@scripts) {    
            test_script($script, 1);
        }
    }
}

##################################################
sub clean {         # Clean up all reference files
##################################################
    my ($self) = @_;
    my $file;

    foreach $test (@{$self->{tests}}) {    
        my ($testname, @scripts) = @$test;

        foreach $script (@scripts) {    
            foreach $file ("$script.ref",
                           "$script.ref.bak") {
                if(-e $file) {
                    print "Cleaning up $file\n";
                    unlink($file) ||  
                        die "Cannot unlink $file";
                }
            }
        }
    }
}

##################################################
sub run {
##################################################
    my $self = shift;

    foreach $test (@{$self->{tests}}) {    
        my ($testname, @scripts) = @$test;
        my $total = @scripts;
        my ($failed, $success) = (0,0);

        foreach $script (@scripts) {    
            if(test_script($script)) {
                $success++;
            } else {
                print "$testname: " .
                      "$script REF mismatch\n";
                $failed++;
            }
        }

        my $dots = "";
        if(length("$testname$total") < 40) {
            $dots = "." x 
             (40 - 2 - length("$testname$total"));
        }

        if($total eq $success) {
           print "$testname ($total) $dots ok\n";
        } else { 
           print "$testname ($total) $dots " . 
                 "$success ok, $failed not ok\n";
        }
    }
}
 
 ##################################################
 sub test_script {
 ##################################################
     my ($script, $create_ref) = @_;
     my $shouldbe;
     my $retval = 0;
 
     my $cwd = Cwd::cwd();      # Get current dir
 
         # Change to dir where script resides in
     my($base, $path) = 
               File::Basename::fileparse($script);
     chdir($path) || die "Chdir $path failed";
 
     open(PIPE, "$base |") || 
         die "Cannot open $script";
     my $output = join('', <PIPE>);
     close(PIPE) || die "Script $script fails";
 
     if(! -f "$base.ref") {
         # REF file doesn't exist
         if($create_ref) {
             open(FILE, ">$base.ref") ||
                 die "Cannot create $script.ref";
             print FILE $output;
             close(FILE);
             print "Created REF $script.ref\n";
         } else {
             die "No reference file for $script";
         }
     } else {
         # REF file does exist, read it
         open(FILE, "<$base.ref") ||
             die "Cannot open $script.ref";
         $shouldbe = join('', <FILE>);
         close(FILE);
 
         if($create_ref) {
             if($output ne $shouldbe) {
 
                 # Create backup
                 File::Copy::copy("$base.ref", 
                                "$base.ref.bak") ||
                     die "Backup $base.ref failed";
 
                 # Write new reference file
                 open(FILE, ">$base.ref") || 
                     die "Cannot write $script.ref";
                 print FILE $shouldbe;
                 close(FILE);
                 print "Updated REF $script.ref\n";
             }
         } else {
             $retval = ($output eq $shouldbe);
         }
     }
     
     chdir($cwd) || die "Cannot chdir to $cwd";
 
     return($retval);
 }
 
 1;

