#!/usr/bin/perl -w
##################################################
# XmlTestSuite - mschilli@perlmeister.com, 2000
##################################################

package XmlTestSuite;

use XmlUserAgent;
use HTTP::Cookies;
use XML::Parser;
use Cwd;
use IO::File;
use strict;

use constant COLUMN_WIDTH => 40;

##################################################
# Constructor
##################################################
sub new {
    my ($class) = @_;
    my $self = { ok => 0, notok => 0 };
    bless $self, $class;
}

##################################################
# Parse the XML test suite file
##################################################
sub parsefile {
    my ($self, $file) = @_;

    my $start_handler = sub { 
                   $self->parse_handle_start(@_) };
    my $end_handler = sub { 
                   $self->parse_handle_end(@_) };

    $self->{parser} = XML::Parser->new( 
            Handlers => { Start => $start_handler,
                          End   => $end_handler });
    return $self->{parser}->parsefile($file);
}

##################################################
# Run test cases
##################################################
sub run {
    my $self = shift;
    local $| = 1;

    if(exists $self->{logfile}) {
        $self->{log} = 
               IO::File->new(">$self->{logfile}");
        die "Cannot open logfile $self->{logfile}" 
            unless defined $self->{log};
    }

    foreach my $case (@{$self->{cases}}) {
        my $cwd = Cwd::cwd();    # Get current dir

        if(exists $case->{dir}) {
            chdir $case->{dir} or 
               die "Cannot chdir to $case->{dir}";
        }

        print_headline($case->{name});

        if($self->handle_case($case)) {
            print "ok\n";
            $self->{ok}++;
            $self->log("$case->{name} ok");
        } else {
            print "not ok\n";
            $self->{notok}++;
            $self->log("$case->{name} not ok");
        }

        chdir($cwd) if $case->{dir};
    }

    $self->{log}->close if exists $self->{logfile};

    print "$self->{ok} tests OK ", $self->{notok} ? 
        ", $self->{notok} FAILED" : "", "\n";
}

##################################################
# Print test headline
##################################################
sub print_headline {
    my $name = shift;
    my $dots = "";

    print "$name ";
    if(length($name) < COLUMN_WIDTH) {
        $dots = "." x 
             (COLUMN_WIDTH - 2 - length($name));
    }
    print "$dots ";
}
 
 ##################################################
 # Handle one test case
 ##################################################
 sub handle_case {
     my ($self, $case) = @_;
     my $data;
 
     if(exists $case->{requests}) {
         my $resp = $self->fetch($case->{requests});
 
         if($resp->is_success) {
             $data = $resp->content();
         } else {
             print "Error: ", $resp->code, " - ", 
             $resp->message(), "\n";
         }
     }
 
     if(exists $case->{function}) {
         open PIPE, "$case->{function} |" or 
             die "Cannot open $case->{function}";
         $data = join '', <PIPE>;
         close PIPE;
     }
 
     if(exists $case->{regex}) {
         if(eval "\$data =~ $case->{regex}") {
             return 1;
         } else  {
             $self->log("$case->{name}: No match ",
              "for $case->{regex}, got:\n$data\n");
             return 0;
         }
     }
 
     if(exists $case->{reffile}) {
         my $refdata = suckfile($case->{reffile});
         if(defined $refdata) {
             return 1 if $refdata eq $data;
 
             $self->log("$case->{name}: mismatch" .
             " $case->{reffile}, got \n$data\n");
             return 0;
         } else {
             dumptofile($case->{reffile}, $data);
             print "CREATED $case->{reffile} ";
             return 1;
         }
     }
 
     if($case->{filter}) {
         open PIPE, "| $case->{filter}" or 
             die "Cannot open filter";
         print PIPE $data;
         close PIPE;
     }
 
     return 1;
 }
 
 ##################################################
 sub parse_handle_start {
 ##################################################
     my ($self, $p, $el, %atts) = @_;
 
     if($el eq "testcase") {
         $self->{case}  = { name => $atts{name},
                            dir  => $atts{dir} };
     }
 
     if($el eq "log") {
         $self->{logfile} = $atts{file};
     }
 
     if($el eq "request") {
         if(exists $atts{xml}) {
             push(@{$self->{case}->{requests}}, 
                 [$atts{xml}, $atts{fixup} || ""]);
         } elsif(exists $atts{function}) {
             $self->{case}->{function} = 
                                   $atts{function};
         }
     }
 
     if($el eq "function") {
         $self->{case}->{function} = $atts{"name"};
     }
 
     if($el eq "expect") {
         $self->{case}->{regex} = $atts{regex} if
             exists $atts{regex};
         $self->{case}->{reffile} = $atts{reffile} 
             if exists $atts{reffile};
     }
 
     if($el eq "print") {
         $self->{case}->{filter} = $atts{filter};
     }
 }
 
 ##################################################
 sub parse_handle_end {
 ##################################################
     my ($self, $p, $el, %atts) = @_;
 
     if($el eq "testcase") {
         push(@{$self->{cases}}, $self->{case});
     }
 }
 
 ##################################################
 sub log {
 ##################################################
     my ($self, $msg) = @_;
 
     return unless exists $self->{log};
 
     my ($sec,$min,$hour,$mday,$mon,$year) = 
                                   localtime(time);
     my $fh = $self->{log};
 
     printf $fh "%4d/%02d/%02d %02d:%02d:%02d> " .
            "%s\n", $year + 1900, $mon+1, $mday, 
            $hour, $min, $sec, $msg;
 }
 
 ##################################################
 sub fetch {
 ##################################################
     my ($self, $requests) = @_;
     my $resp;
 
     my $ua  = XmlUserAgent->new();
     my $jar = HTTP::Cookies->new();
 
     $ua->cookie_jar($jar);
 
     foreach my $req (@$requests) {
         my($xmlfile, $fixup) = @$req;
 
         my $data = suckfile($xmlfile) or 
                        die "Cannot open $xmlfile";
         $resp = $ua->request($data);
 
         {  if($resp->code == 302) {
                $req = HTTP::Request->new(GET => 
                    $resp->header('Location'));    
                $resp = $ua->request($req);
                redo;
            }
 
            if($resp->is_success && 
               $resp->content =~ 
                 /<META.*?Refresh.*?URL=(.*?)"/s) {
                $req = HTTP::Request->new(GET => $1);    
                $resp = $ua->req($req);
                redo;
            }
         }
 
         if($fixup and $resp->is_success) {
             open FIXUP, "| $fixup" or 
                          die "Cannot open $fixup";
             print FIXUP $resp->content;
             close FIXUP or 
                        die "Error running $fixup";
         }
     }
 
     return $resp;
 }
 
 ##################################################
 sub suckfile {
 ##################################################
     my $name = shift;
 
     open FILE, "<$name" or return undef;
     my $data = join '', <FILE>;
     close FILE;
 
     $data;
 }
 
 ##################################################
 sub dumptofile {
 ##################################################
     my ($name, $data) = @_;
 
     open FILE, ">$name" or die "Can't open $name";
     print FILE $data;
     close FILE;
 }
 
 1;

